=begin
 copyright(c) 2007 arton

 Usage of the works is permitted provided that this instrument is retained with the works, 
 so that any entity that uses the works is notified of this instrument. 

 DISCLAIMER: THE WORKS ARE WITHOUT WARRANTY. 

 [2004, Fair License: rhid.com/fair] 
=end

require 'mscorlib.dll'
require 'win32ole/win32ole.net.dll'

class WIN32OLE
  VERSION = '0.7.2'
  ARGV = []
  CP_ACP = 0
  CP_OEMCP = 1
  CP_MACCP = 2
  CP_THREAD_ACP = 3
  CP_SYMBOL = 42
  CP_UTF7 = 65000
  CP_UTF8 = 65001

  @@code_page = CP_ACP

  def initialize(progid)
    if Win32olenet::WIN32OLE === progid
      @win32ole = progid
    else
      @win32ole = Win32olenet::WIN32OLE.new(progid)
    end
  end

  def self.connect(svr_name, *others)
    raise NotImplementedError, 'connect'
  end

  def self.const_load()
    raise NotImplementedError, 'connect'
  end

  def self.ole_free(obj)
    if WIN32OLE === obj
      obj.ole_release
    else
      0
    end
  end

  def self.ole_shutdown
  end

  def self.ole_reference_count(obj)
    if WIN32OLE === obj
      obj.ole_reference_count
    end
  end

  def self.ole_show_help(obj, context = nil)
    raise NotImplementedError, 'ole_show_help'
  end

  def self.codepage
    @@code_page
  end

  def self.codepage=(val)
    @@code_page = val
  end

  def method_missing(id, *args)
    name = id.to_s
    if name[-1] == ?=
      self[name[0..-2]] = *args
    else
      invoke name, *args
    end
  end

  def invoke(name, *args)
    if args
      if Hash === args.last
        hash = args.delete_at -1
        o = _named_args_call name, args, hash
      else
        o = @win32ole.Invoke name, args
      end
    else
      o = @win32ole.Invoke name, []
    end
    if Win32olenet::WIN32OLE === o
      WIN32OLE.new o
    else
      o
    end
  end

  def [](id)
    @win32ole.Invoke id, []
  end

  def ole_free
    @win32ole.Release
  end

  def each
    enum = @win32ole.GetEnumerator
    while enum.MoveNext
      yield enum.Current
    end
    enum.Dispose
  end

  def setproperty(name, *val)
    @win32ole.SetProperty name, val
  end

  alias []= setproperty

  def _invoke(dispid, args, types)
    raise NotImplementedError, '_invoke'
  end

  def _getproperty(dispid, args, types)
    raise NotImplementedError, '_getproperty'
  end

  def _setproperty(dispid, args, types)
    raise NotImplementedError, '_setproperty'
  end

  def ole_methods
  end

  def ole_get_methods
  end

  def ole_put_methods
  end

  def ole_func_methods
  end

  def ole_method(cmdname)
  end
  alias ole_method_help ole_method

  def ole_obj_help
  end

  module VARIANT
    VT_I2 = 2
    VT_I4   = 3
    VT_R4   = 4
    VT_R8   = 5
    VT_CY   = 6
    VT_DATE = 7
    VT_BSTR = 8
    VT_DISPATCH     = 9
    VT_ERROR        = 10
    VT_BOOL = 11
    VT_VARIANT      = 12
    VT_UNKNOWN      = 13
    VT_DECIMAL      = 14
    VT_I1   = 16
    VT_UI1  = 17
    VT_UI2  = 18
    VT_UI4  = 19
    VT_I8   = 20
    VT_UI8  = 21
    VT_INT  = 22
    VT_UINT = 23
    VT_PTR  = 26
    VT_USERDEFINED  = 29
    VT_ARRAY        = 0x2000
    VT_BYREF        = 0x4000
  end

  protected

  def ole_reference_count
    @win32ole.ReferenceCount
  end

  def ole_release
    @win32ole.Release
  end

  attr_reader :win32ole

  def _named_args_call(name, args, kv)
    hash = System::Collections::Hashtable.new
    kv.each do |k, v|
      if WIN32OLE === v
        hash.Add k.to_s, v.win32ole
      else
        hash.Add k.to_s, v
      end
    end
    @win32ole.InvokeWithNamedArguments name, args, hash
  end
end

class WIN32OLE_TYPE
  def self.ole_classes(typelib)
  end

  def self.typelibs
  end

  def self.progids
  end

  def initialize()
    @name = nil
    @ole_type = nil
    @guid = nil
    @progid = nil
    @visible = nil
    @major_version = nil
    @minor_version = nil
    @typekind = nil
    @helpstring = nil
    @src_type = nil
    @helpfile = nil
    @helpcontext = nil
  end

  attr_reader :name, :ole_type, :guid, :progid, :major_version, :minor_version, :typekind,
          :helpstring, :src_type, :helpfile, :helpcontext
  alias to_s name
  def visible?
    @visible
  end

  def variables
    raise NotImplementedError, 'variables'
  end
  def ole_methods(*args)
    raise NotImplementedError, 'ole_methods'
  end
end

class WIN32OLE_VARIABLE

end

class WIN32OLE_METHOD
end

class WIN32OLE_PARAM
end

class WIN32OLE_EVENT
  def initialize
  end
  def on_event
  end
  def on_event_with_outargs
  end
end

class WIN32OLERuntimeError < RuntimeError
end
